from django.core.cache import cache
from functools import wraps
import hashlib
import json


def __sha224(data):
    return hashlib.sha224(data.encode("utf-8")).hexdigest()


def api_cache_global(cache_timeout):
    def _api_cache(func):  # pragma: no cover
        def cache_logic(*argv, **kwargs):
            response = None
            try:
                func_name = func.__name__
                cache_key_args = {"args": argv[2:], "kwargs": kwargs}
                cache_key = __sha224(json.dumps(cache_key_args))
                cached_data = cache.get(func_name)
                if cached_data is None:
                    cached_data = {}
                if cache_key not in cached_data:
                    response = func(*argv, **kwargs)
                    if "status" in response and response["status"] == 200:
                        cached_data[cache_key] = response
                        cache.set(func_name, cached_data, cache_timeout)
                else:
                    response = cached_data[cache_key]
            except ConnectionError:
                response = func(*argv, **kwargs)
            finally:
                return response
        return wraps(func)(cache_logic)
    return _api_cache



from rest_framework.viewsets import GenericViewSet
from rest_framework.serializers import Serializer
from rest_framework.response import Response
from nimblehelper.helper import NimbleHelper


class BaseView(GenericViewSet):
    @staticmethod
    def __placeholder_function(x_consumer_id, params):
        return {'status': 500, 'x_consumer_id': x_consumer_id, 'params': params}

    serializer_class = Serializer

    list_api_function = __placeholder_function
    list_fields = []
    list_required_fields = []

    create_api_function = __placeholder_function
    create_fields = []
    create_required_fields = []

    retrieve_api_function = __placeholder_function
    retrieve_fields = []
    retrieve_required_fields = []

    @classmethod
    def list(cls, request):
        params = NimbleHelper.check_get_parameters(request=request, fields=cls.list_fields,
                                                   required_fields=cls.list_required_fields)
        response = cls.list_api_function(x_consumer_id=params["x_consumer_id"],
                                         params=params["data"])
        return Response(response, status=response["status"])

    @classmethod
    def create(cls, request):
        params = NimbleHelper.check_post_parameters(request=request, fields=cls.create_fields,
                                                    required_fields=cls.create_required_fields)
        response = cls.create_api_function(x_consumer_id=params["x_consumer_id"],
                                           params=params["data"])
        return Response(response, status=response["status"])

    @classmethod
    def retrieve(cls, request, pk):
        params = NimbleHelper.check_get_parameters(request=request, fields=cls.retrieve_fields,
                                                   required_fields=cls.retrieve_required_fields, pk=pk)
        response = cls.retrieve_api_function(x_consumer_id=params['x_consumer_id'], params=params['data'])
        return Response(response, status=response["status"])



from rest_framework.viewsets import GenericViewSet
from rest_framework.response import Response
from django.contrib.auth.models import User


class HealthViewSet(GenericViewSet):
    """
    API that checks db status (checks server status if able to reach)
    """
    def list(self, request):
        status = "UP"
        try:
            if request.GET.get('status', None):
                status = "UP"
            self.__test_creating_user()
            db_status = "UP"
            status_code = 200
        except ConnectionError:
            db_status = "DOWN"
            status_code = 500
        health_data = {
            "dbStatus": db_status,
            "serviceStatus": status
        }
        return Response(health_data, status=status_code)

    @staticmethod
    def __test_creating_user():
        test_db_user = User.objects.create(username="FooBar")
        test_db_user.save()
        test_db_user.delete()



from .settings import SUCCESSFUL, INTERNAL_SERVER_ERROR, CODE_LIST
import hashlib


class NimbleCodes:
    @classmethod
    def http_code_helper(cls, code, message=None, data=None):  # pragma: no
        if code not in CODE_LIST:
            code = 500
        response_code_list = {
            200: cls.__200_success(code=code, data=data),
            201: cls.__200_success(code=code, data=data),
            400: cls.__400_error_message(code=code, message=message),
            401: cls.__400_error_message(code=code, message=message),
            403: cls.__400_error_message(code=code, message=message),
            500: cls.__500_error_message(code=code, data=data, message=message)
        }
        return response_code_list[code]

    @staticmethod
    def sha224(data):
        return hashlib.sha224(data.encode("utf-8")).hexdigest()

    @staticmethod
    def __200_success(code, data, message=SUCCESSFUL):  # pragma: no cover
        data = [] if data is None else data
        return {"status": code, "message": message, "data": data}

    @staticmethod
    def __400_error_message(code, message):  # pragma: no cover
        return {"status": code, "message": message, "data": []}

    @staticmethod
    def __500_error_message(code, data, message=INTERNAL_SERVER_ERROR):  # pragma: no cover
        data = [] if data is None else data
        return {"status": code, "message": message, "data": data}


from requests import post, get
from .settings import KONG_ADMIN_URL


def login(url, username, password):
    data = {"username": username, "password": password}
    return post(url=url, data=data)


def get_x_consumer_id(username):
    url = KONG_ADMIN_URL + "consumers/" + username + "/"
    return get(url=url)



from .exceptions import BadInputException, InvalidAuthentication, InternalServerError
from .code import NimbleCodes
from .settings import SIEBEL_GATEWAY_URL, SUCCESS_CODES
from django.http import QueryDict
import requests
import json


# This class was created in order to extract functionality from nimble microservices
class NimbleHelper:
    @staticmethod
    def cut_response_hierarchy(hierarchy, response):
        hierarchy_list = hierarchy.split('.')
        try:
            for hierarchy_level in hierarchy_list:
                if hierarchy_level == '0':
                    response = response[int(hierarchy_level)]
                else:
                    response = response[hierarchy_level]
        except (KeyError, TypeError):
            response = []
        return response

    # Checks Request For HTTP_X_CONSUMER_ID header (indicates kong has authorized the request)
    @staticmethod
    def check_authorization(request):
        x_consumer_id = request.META.get('HTTP_X_CONSUMER_ID', None)
        if not x_consumer_id:
            raise InvalidAuthentication
        return x_consumer_id

    @classmethod
    def run_siebel_request(cls, x_consumer_id, params, soap_action):
        data = json.dumps({"soapaction": soap_action, "data": params})
        headers = {"X-Consumer-ID": x_consumer_id, "Content-Type": "application/json"}
        try:
            results = requests.post(SIEBEL_GATEWAY_URL, data=data, headers=headers)
        except ConnectionError as exception:
            raise InternalServerError(str(exception))
        return json.loads(results.content.decode("utf-8"))

    # Checks the required fields in a tuple [('name',), ('id',)] will be checked in a "AND" format,
    # if multiple values in a tuple e.g. [('name', 'id',)] these will be treated as 'OR' conditionals
    @staticmethod
    def check_required_input(required_fields, call_values):
        try:
            for fields in required_fields:
                check = False
                for field in fields:
                    if call_values['data'][field]:
                        check = True
                if not check:
                    raise BadInputException
        except KeyError:
            raise BadInputException
        return True

    @classmethod
    def request_manager(cls, soap_action, params, x_consumer_id, hierarchy):
        response = cls.run_siebel_request(x_consumer_id=x_consumer_id, params=params, soap_action=soap_action)
        if response['status'] in SUCCESS_CODES:
            response = cls.cut_response_hierarchy(hierarchy=hierarchy, response=response)
            return NimbleCodes.http_code_helper(code=200, data=response)
        else:
            return NimbleCodes.http_code_helper(code=response["status"], message=response["message"])

    # These functions get data from a request passed through
    # ==============================================================================================
    # START OF FUNCTIONS
    # ==============================================================================================
    # Takes in a fields argument (list of tuples) in order to map request variables to a dict, pass
    # through required_fields in order to have required fields add the names as tuples to the required_fields list

    # USED FOR GET REQUESTS
    @classmethod
    def check_get_parameters(cls, request, fields, required_fields=None, pk=None):
        call_values = {
            'x_consumer_id': cls.check_authorization(request),
            'data': {}
        }
        for field in fields:
            parameter, value = field
            if pk and value == "pk":
                call_values['data'][parameter] = pk
            else:
                call_values['data'][parameter] = request.GET.get(value, None)
        cls.check_required_input(required_fields, call_values)
        return call_values

    # USED FOR POST REQUESTS
    @classmethod
    def check_post_parameters(cls, request, fields, required_fields):
        call_values = {
            'x_consumer_id': cls.check_authorization(request),
            'data': {}
        }
        for field in fields:
            parameter, value = field
            call_values['data'][parameter] = request.POST.get(value, None)
        cls.check_required_input(required_fields, call_values)
        return call_values

    # USED FOR PUT REQUESTS
    @classmethod
    def check_put_parameters(cls, request, fields, required_fields, pk):
        call_values = {
            'x_consumer_id': cls.check_authorization(request),
            'data': {}
        }
        put = QueryDict(request.body)
        for field in fields:
            parameter, value = field
            if pk and value == "pk":
                call_values['data'][parameter] = pk
            else:
                call_values['data'][parameter] = put.get(value, None)
        cls.check_required_input(required_fields, call_values)
        return call_values
    # ==============================================================================================
    # END OF FUNCTIONS
    # ==============================================================================================



from setuptools import setup

setup(
    name='nimblehelper',
    version='0.6.25',
    description='Nimble Infrastructure Helper',
    author='Brendan Kamp',
    author_email='brendan@tangentsolutions.co.za',
    license='MIT',
    packages=['nimblehelper'],
    install_requires=[
        'djangorestframework==3.3.2',
        'requests',
        'django==1.9.2'
    ],
    zip_safe=False
)
